package com.example;

import org.springframework.batch.core.*;
import org.springframework.batch.core.job.builder.JobBuilder;
import org.springframework.batch.core.repository.JobRepository;
import org.springframework.batch.core.step.builder.StepBuilder;
import org.springframework.batch.item.ExecutionContext;
import org.springframework.batch.item.support.AbstractItemStreamItemReader;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.transaction.PlatformTransactionManager;

import java.util.List;

@Configuration
@EnableConfigurationProperties(Config.Properties.class)
public class Config {

    @ConfigurationProperties("com.example")
    public record Properties(List<String> stream) {  }

    @Bean
    public Job restartableJob(
            final Properties properties,
            final JobRepository jobRepository,
            final PlatformTransactionManager platformTransactionManager
    ) {
        final String KEY = "state.next";
        return new JobBuilder("restartable-job", jobRepository)
                .start(new StepBuilder("consume", jobRepository)
                        .allowStartIfComplete(true)
                        .<String, String>chunk(100, platformTransactionManager)
                        .reader(new AbstractItemStreamItemReader<>() {

                            private int next = 0;

                            @Override
                            public void open(ExecutionContext executionContext) {
                                // if this is not the first execution, start reading from where we left off
                                if (executionContext.containsKey(KEY)) {
                                    next = executionContext.getInt(KEY);
                                }
                            }

                            @Override
                            public String read() {
                                if (next >= properties.stream().size()) {
                                    return null;
                                }
                                return properties.stream().get(next++);
                            }

                            @Override
                            public void update(ExecutionContext executionContext) {
                                // remember where we left off
                                executionContext.putInt(KEY, next);
                            }

                        })
                        .writer(chunk -> System.out.println(chunk.getItems()))
                        .listener(new StepExecutionListener() {
                            @Override
                            public void beforeStep(StepExecution stepExecution) {
                                // see if we have some state from previous executions and restore it
                                stepExecution.getExecutionContext().put(
                                        KEY,
                                        stepExecution.getJobExecution().getExecutionContext().get(KEY)
                                );
                            }

                            @Override
                            public ExitStatus afterStep(StepExecution stepExecution) {
                                // Move the relevant state to the job's context so that we can reload it on subsequent
                                // executions of this step, as out-of-the=box, such executions (ran after a `COMPLETED`
                                // one) are started with just the job context.
                                stepExecution.getJobExecution().getExecutionContext().put(
                                        KEY,
                                        stepExecution.getExecutionContext().get(KEY)
                                );
                                return null;
                            }
                        })
                        .build())
                .listener(new JobExecutionListener() {
                    @Override
                    public void afterJob(JobExecution jobExecution) {
                        if (jobExecution.getStatus().equals(BatchStatus.COMPLETED)) {
                            // allow this job instance to start again at a later point
                            jobExecution.setStatus(BatchStatus.STOPPED);
                        }
                    }
                })
                .build();
    }

}
